/*
    This file (and dvd.c) also includes DOXYGEN (www.doxygen.com) compatible documentation!

    --------------------------------------------------------------------------------------
    Version    Date              Author                        Log
    --------------------------------------------------------------------------------------
    0.0        21st June 2003    Peter (www.console-dev.de)    Inital Release
    0.1        3rd  July 2003    Peter (www.console-dev.de)    Lots of new functions added!
                                                               Thanks to yursoft.com for his
                                                               source what helped to design this
                                                               document.

*/
/*+----------------------------------------------------------------------------------------------+*/
#ifndef DVD_H
#define DVD_H
/*+----------------------------------------------------------------------------------------------+*/
#include <mygcn.h>
/*+----------------------------------------------------------------------------------------------+*/
#define MEM_DVD_BASE                   (0xCC006000)   ///< Memory address of the DVD Interface
/*+----------------------------------------------------------------------------------------------+*/
#define R_DVD_STATUS_1                 *(vu32*)(MEM_DVD_BASE)        ///< DVD Status1 register
#define R_DVD_STATUS_2                 *(vu32*)(MEM_DVD_BASE+0x04)   ///< DVD Status2 register
#define R_DVD_COMMAND                  *(vu32*)(MEM_DVD_BASE+0x08)   ///< DVD Command register
#define R_DVD_SUBCOMMAND               *(vu32*)(MEM_DVD_BASE+0x09)   ///< DVD SubCommand register
#define R_DVD_SUBCOMMAND2              *(vu32*)(MEM_DVD_BASE+0x0A)   ///< DVD SubCommand2 register
#define R_DVD_OFFSET                   *(vu32*)(MEM_DVD_BASE+0x0C)   ///< DVD Offset register
#define R_DVD_SRCLENGTH                *(vu32*)(MEM_DVD_BASE+0x10)   ///< DVD SourceLength register
#define R_DVD_DSTADDRESS               *(vu32*)(MEM_DVD_BASE+0x14)   ///< DVD Destination Buffer (address) register
#define R_DVD_DSTLENGTH                *(vu32*)(MEM_DVD_BASE+0x18)   ///< DVD Destination Buffer Length register
#define R_DVD_ACTION                   *(vu32*)(MEM_DVD_BASE+0x1C)   ///< DVD Action register
#define R_DVD_ERRORCODE                *(vu32*)(MEM_DVD_BASE+0x20)   ///< DVD ErrorCode register
#define R_DVD_CONFIG                   *(vu32*)(MEM_DVD_BASE+0x24)   ///< DVD COnfig registr
/*+----------------------------------------------------------------------------------------------+*/
#define DVD_COMMAND_INQUIRY            (0x12)   ///< Can be used with DVD_SetCommand()
#define DVD_COMMAND_STOP               (0xE3)   ///< Can be used with DVD_SetCommand()
#define DVD_COMMAND_READ               (0xA8)   ///< Can be used with DVD_SetCommand()
#define DVD_COMMAND_SEEK               (0xAB)   ///< Can be used with DVD_SetCommand()
#define DVD_COMMAND_REQUESTERROR       (0xE0)   ///< Can be used with DVD_SetCommand()
#define DVD_COMMAND_AUDIOSTREAM        (0xE1)   ///< Can be used with DVD_SetCommand()
#define DVD_COMMAND_AUDIOSTATUS        (0xE2)   ///< Can be used with DVD_SetCommand()
#define DVD_COMMAND_AUDIO              (0xE4)   ///< Can be used with DVD_SetCommand()
/*+----------------------------------------------------------------------------------------------+*/
#define M_DVD_SET_SRCLENGTH(n)         R_DVD_SRCLENGTH = (u32)n;     ///< Set dvd sourcelength register
#define M_DVD_SET_STATUS_1(n)          R_DVD_STATUS_1 = (u32)(n);    ///< Set dvd status1 register
#define M_DVD_SET_STATUS_2(n)          R_DVD_STATUS_2 = (u32)(n);    ///< Set dvd status2 register
#define M_DVD_SET_OFFSET(n)            R_DVD_OFFSET = (u32)n;        ///< Set dvd offset register
#define M_DVD_SET_DSTADDRESS(n)        R_DVD_DSTADDRESS = (u32)(n);  ///< Set dvd destination address register
#define M_DVD_SET_DSTLENGTH(n)         R_DVD_DSTLENGTH = (u32)(n);   ///< Set dvd destination length register
#define M_DVD_SET_ACTION(n)            R_DVD_ACTION = (u32)(n);      ///< Set dvd action register
/*+----------------------------------------------------------------------------------------------+*/
#define DVD_STATE_COVER_OPEN           (0x05)   ///< Internally used by DVD_IsCoverOpen()
/*+----------------------------------------------------------------------------------------------+*/
#define DVD_AUDIO_DISABLE              (0)      ///< Helper define. Can be used with DVD_SetAudio()
#define DVD_AUDIO_ENABLE               (1)      ///< Helper define. Can be used with DVD_SetAudio()
/*+----------------------------------------------------------------------------------------------+*/
#define DVD_STATUS_1                   (1)      ///< Helper define. Can be used with DVD_GetStatus()
#define DVD_STATUS_2                   (2)      ///< Helper define. Can be used with DVD_GetStatus()
#define DVD_SEEK_SET                   (0)      ///< Helper define. Can be used with DVD_Seek()
#define DVD_SEEK_CUR                   (1)      ///< Helper define. Can be used with DVD_Seek()
/*+----------------------------------------------------------------------------------------------+*/
#define DVD_SECTORLENGTH               (2048)   ///< DVD Sector Length. Internally used by many functions
/*+----------------------------------------------------------------------------------------------+*/
/*!
\brief Struture for DVD info

This can be used with DVD_GetDriveInfo()
*/
typedef struct
{
  u16 RevisionLevel;
  u16 DeviceCode;
  u32 ReleaseDate;
  u8  Padding[24];
} DVD_DRIVEINFO;
/*+----------------------------------------------------------------------------------------------+*/
#ifdef __cplusplus
   extern "C" {
#endif /* __cplusplus */
/*+----------------------------------------------------------------------------------------------+*/
extern u8 DVD_IsCoverOpen(void);
extern void DVD_Stop(void);
extern u32 DVD_GetCurrentOffset(void);
extern u32 DVD_GetLastCommands(void);
extern u32 DVD_GetCurrentSector(void);
extern void DVD_ReadSector(void *pDstBuffer, u32 Sector);
extern void DVD_Read(void *pDstBuffer, u32 Count, u32 Offset);
extern void DVD_Seek(u32 Offset, u8 Whence);
extern void DVD_SetCommand(u8 Command);
extern void DVD_SetSubCommand(u8 SubCommand);
extern void DVD_SetSubCommand2(u16 SubCommand2);
extern void DVD_SetCommands(u8 Command, u8 SubCommand, u16 SubCommand2);
extern u32 DVD_GetStatus(u8 StatusNo);

extern void DVD_GetDriveInfo(DVD_DRIVEINFO *pDVDDriveInfo);
extern void DVD_SetAudio(u8 Value);
extern u32 DVD_GetErrorCode(void);
extern void DVD_RequestError(void);
extern void DVD_Init(void);
/*+----------------------------------------------------------------------------------------------+*/
#ifdef __cplusplus
   }
#endif /* __cplusplus */
#endif /* DVD_H */

/* END OF FILE */
