#include "debug.h"
#include "debugfont.raw.c"
/*+----------------------------------------------------------------------------------------------+*/
/*!
\addtogroup debug_functions Debug Functions
*/
/*+----------------------------------------------------------------------------------------------+*/
#ifndef DOXYGEN_SHOULD_SKIP_THIS
static u32 *s_pFrameBuffer;
static u32 s_Color;
static u8 *s_pLetterPointer[256];
static char s_Abc[95]=
"_!\"#$\%&'()*+,-./0123456789:;<=>?@abcdefghijklmnopqrstuvwxyz[]ABCDEFGHIJKLMNOPQRSTUVWXYZ+|";
#endif /* DOXYGEN_SHOULD_SKIP_THIS */
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup debug_functions
\brief Set debugtext-system framebuffer address

\param pFrameBuffer
   Pointer to the framebuffer, where the debug textsystem shall write to.

*/
/*
   Author: Peter (www.console-dev.de)
   Date..: 20th June 2003
*/
void DEBUG_SetFrameBuffer(u32 *pFrameBuffer)
{
   s_pFrameBuffer = pFrameBuffer;
}
/*+----------------------------------------------------------------------------------------------+*/
#ifndef DOXYGEN_SHOULD_SKIP_THIS
// Helper function for DEBUG_Print stuff
int StrLen(char *pText)
{
   int iCnt=0;

   while(*pText++ != '\0') iCnt++;

   return iCnt;
}
#endif /* DOXYGEN_SHOULD_SKIP_THIS */
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup debug_functions
\brief Set debugtext-system font color

\param Color
   Debug-font-color. Specify it as YCbYCr value.

\sa
   COLOR_ConvertRgb2YCbYCr()
*/
/*
   Author: Peter (www.console-dev.de)
   Date..: 20th June 2003
*/
void DEBUG_SetTextColor(u32 Color)
{
   s_Color = Color;
}
/*+----------------------------------------------------------------------------------------------+*/
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup debug_functions
\brief Init debug-onscreen-text-system

\param pFrameBuffer
   Pointer to the framebuffer, where the debug textsystem shall write to.

\param Color
   Debug-font-color. Specify it as YCbYCr value.

\sa
   DEBUG_Print()           \n
   DEBUG_ShowValueU8()     \n
   DEBUG_ShowValueU16()    \n
   DEBUG_ShowValueU32()    \n
*/
/*
   Author: Peter (www.console-dev.de)
   Date..: 20th June 2003
*/
void DEBUG_InitText(u32 *pFrameBuffer, u32 Color)
{
   int i;
   int iAbcLen=sizeof(s_Abc);

   for(i=0; i<256; i++)
   {
      s_pLetterPointer[i] = NULL;
   }

   for(i=0; i<iAbcLen; i++)
   {
      if(NULL == s_pLetterPointer[(u8)s_Abc[i]])
      {
         s_pLetterPointer[(u8)s_Abc[i]] = (u8*)&DEBUG_FontBitmap[DEBUG_TEXT_CHAR_SIZE*i];
      }
   }

   // all unsupported chars point to the space character
   for(i=0; i<256; i++)
   {
      if(NULL == s_pLetterPointer[i])
      {
         s_pLetterPointer[i] = s_pLetterPointer[' '];
      }
   }

   s_pLetterPointer[0x0] = (u8*)&DEBUG_FontBitmap[DEBUG_TEXT_CHAR_SIZE*(16)]; // 0
   s_pLetterPointer[0x1] = (u8*)&DEBUG_FontBitmap[DEBUG_TEXT_CHAR_SIZE*(17)]; // 1
   s_pLetterPointer[0x2] = (u8*)&DEBUG_FontBitmap[DEBUG_TEXT_CHAR_SIZE*(18)]; // 2
   s_pLetterPointer[0x3] = (u8*)&DEBUG_FontBitmap[DEBUG_TEXT_CHAR_SIZE*(19)]; // 3
   s_pLetterPointer[0x4] = (u8*)&DEBUG_FontBitmap[DEBUG_TEXT_CHAR_SIZE*(20)]; // 4
   s_pLetterPointer[0x5] = (u8*)&DEBUG_FontBitmap[DEBUG_TEXT_CHAR_SIZE*(21)]; // 5
   s_pLetterPointer[0x6] = (u8*)&DEBUG_FontBitmap[DEBUG_TEXT_CHAR_SIZE*(22)]; // 6
   s_pLetterPointer[0x7] = (u8*)&DEBUG_FontBitmap[DEBUG_TEXT_CHAR_SIZE*(23)]; // 7
   s_pLetterPointer[0x8] = (u8*)&DEBUG_FontBitmap[DEBUG_TEXT_CHAR_SIZE*(24)]; // 8
   s_pLetterPointer[0x9] = (u8*)&DEBUG_FontBitmap[DEBUG_TEXT_CHAR_SIZE*(25)]; // 9

   s_pLetterPointer[0xa] = (u8*)&DEBUG_FontBitmap[DEBUG_TEXT_CHAR_SIZE*(33)]; // a
   s_pLetterPointer[0xb] = (u8*)&DEBUG_FontBitmap[DEBUG_TEXT_CHAR_SIZE*(34)]; // b
   s_pLetterPointer[0xc] = (u8*)&DEBUG_FontBitmap[DEBUG_TEXT_CHAR_SIZE*(35)]; // c
   s_pLetterPointer[0xd] = (u8*)&DEBUG_FontBitmap[DEBUG_TEXT_CHAR_SIZE*(36)]; // d
   s_pLetterPointer[0xe] = (u8*)&DEBUG_FontBitmap[DEBUG_TEXT_CHAR_SIZE*(37)]; // e
   s_pLetterPointer[0xf] = (u8*)&DEBUG_FontBitmap[DEBUG_TEXT_CHAR_SIZE*(38)]; // f

   DEBUG_SetFrameBuffer (pFrameBuffer);
   DEBUG_SetTextColor   (Color);
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup debug_functions
\brief Show an u8 on screen

\param x
   Horizontal Screen Coordinate

\param y
   Vertical Screen Coordinate

\param ValueToShow
   The 8bit value you want to display on screen

\sa
   DEBUG_Print()           \n
   DEBUG_ShowValueU16()    \n
   DEBUG_ShowValueU32()    \n
*/
/*
   Author: Peter (www.console-dev.de)
   Date..: 20th June 2003
*/
void DEBUG_ShowValueU8(short x, short y, u8 ValueToShow)
{
   int xx,yy, act=0;
   u8 *pLetter=NULL;
   u8 _1, _2;

   ValueToShow &= 0xff;

   _1 = (u8)(ValueToShow>>4);
   _2 = (u8)(ValueToShow&0xf);

   pLetter = s_pLetterPointer[_1];

   for(yy=0; yy<DEBUG_TEXT_CHAR_HEIGHT; yy++)
   {
      for(xx=0; xx<DEBUG_TEXT_CHAR_WIDTH; xx++)
      {
         if(0 != pLetter[act])
         {
            s_pFrameBuffer[(320*(y+yy))+((xx+x)>>1)] = s_Color;
         }
         act++;
      }
   }

   act=0;
   _1 = (u8)(ValueToShow>>4);
   _2 = (u8)(ValueToShow&0xf);

   pLetter = s_pLetterPointer[_2];

   for(yy=0; yy<DEBUG_TEXT_CHAR_HEIGHT; yy++)
   {
      for(xx=0; xx<DEBUG_TEXT_CHAR_WIDTH; xx++)
      {
         if(0 != pLetter[act])
         {
            s_pFrameBuffer[(320*(y+yy))+((xx+DEBUG_TEXT_CHAR_WIDTH+x)>>1)] = s_Color;
         }
         act++;
      }
   }
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup debug_functions
\brief Show an u16 on screen

\param x
   Horizontal Screen Coordinate

\param y
   Vertical Screen Coordinate

\param ValueToShow
   The 16bit value you want to display on screen

\sa
   DEBUG_Print()           \n
   DEBUG_ShowValueU8()     \n
   DEBUG_ShowValueU32()    \n
*/
/*
   Author: Peter (www.console-dev.de)
   Date..: 20th June 2003
*/
void DEBUG_ShowValueU16(short x, short y, u16 ValueToShow)
{
   u8 _1, _2;

   ValueToShow &= 0xffff;

   _1 = DEBUG_GET_1ST_U8_FROM_U16(ValueToShow);
   _2 = DEBUG_GET_2ND_U8_FROM_U16(ValueToShow);

   DEBUG_ShowValueU8(   x, y, _1);
   DEBUG_ShowValueU8(32+x, y, _2);
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup debug_functions
\brief Show an u32 on screen

\param x
   Horizontal Screen Coordinate

\param y
   Vertical Screen Coordinate

\param ValueToShow
   The 32bit value you want to display on screen

\sa
   DEBUG_Print()           \n
   DEBUG_ShowValueU16()    \n
   DEBUG_ShowValueU32()    \n
*/
/*
   Author: Peter (www.console-dev.de)
   Date..: 20th June 2003
*/
void DEBUG_ShowValueU32(short x, short y, u32 ValueToShow)
{
   u16 _1, _2;

   ValueToShow &= 0xffffffff;

   _1 = DEBUG_GET_1ST_U16_FROM_U32(ValueToShow);
   _2 = DEBUG_GET_2ND_U16_FROM_U32(ValueToShow);

   DEBUG_ShowValueU16(   x, y, _1);
   DEBUG_ShowValueU16(64+x, y, _2);
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup debug_functions
\brief Print a single char to screen

\param x
   Horizontal Screen Coordinate

\param y
   Vertical Screen Coordinate

\param CharToPrint
   The char/letter you want to print

\sa
   DEBUG_Print()
*/
/*
   Author: Peter (www.console-dev.de)
   Date..: 20th June 2003
*/
void DEBUG_PrintChar(short x, short y, char CharToPrint)
{
   int xx,yy, act=0;
   u8 *pLetter=NULL;

   pLetter = s_pLetterPointer[(u8)CharToPrint];

   for(yy=0; yy<DEBUG_TEXT_CHAR_HEIGHT; yy++)
   {
      for(xx=0; xx<DEBUG_TEXT_CHAR_WIDTH; xx++)
      {
         if(0 != pLetter[act])
         {
            s_pFrameBuffer[(320*(y+yy))+((xx+x)>>1)] = s_Color;
         }
         act++;
      }
   }
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup debug_functions
\brief Print text to screen

\param x
   Horizontal Screen Coordinate

\param y
   Vertical Screen Coordinate

\param pText
   Pointer to a NULL-terminated text

\note
   This function is quite <b>slow</b> and should be used only for debug purposes. \n \n

   It supports the following characters: \n
      -  !"#$%&'()*+,-./0123456789:;<=>?@abcdefghijklmnopqrstuvwxyz[]ABCDEFGHIJKLMNOPQRSTUVWXYZ+| \n

   In case you specify an unsupported character, this character will not be displayed.
*/
/*
   Author: Peter (www.console-dev.de)
   Date..: 20th June 2003
*/
void DEBUG_Print(short x, short y, char *pText)
{
   #define MAKE_NEW_LINE iCurX=x; iCurY+=20;

   int i;
   int iCurX=x, iCurY=y;

   for(i=0; i<StrLen(pText); i++)
   {
      iCurX+=DEBUG_TEXT_CHAR_WIDTH;

      // wrap line
      if(iCurX >= 640-DEBUG_TEXT_CHAR_WIDTH)
      {
         MAKE_NEW_LINE;
      }

      // dont display spaces
      if(' ' != pText[i])
      {
         // make newline
         if('\n' == pText[i])
         {
            MAKE_NEW_LINE;
         }
         else
         {
            DEBUG_PrintChar(iCurX /*x+(i*DEBUG_TEXT_CHAR_WIDTH)*/, iCurY, pText[i]);
         }
      }
   }
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup debug_functions
\brief Check an assertion statement

Do not use this. Use ::ASSERT instead !
*/
void DEBUG_Assert(u8 Assertion, char* pMessage, char* pExpression, char* pFileName, u32 LineNo)
{
#ifndef DOXYGEN_SHOULD_SKIP_THIS
   #ifndef REG_HOTRESET
      #define REG_HOTRESET             (0xCC003024)
   #endif /* REG_HOTRESET */
   #ifndef REG_HOTRESET_PTR
      #define REG_HOTRESET_PTR         ((volatile u32*)REG_HOTRESET)
   #endif /* REG_HOTRESET_PTR */
   #ifndef M_HOTRESET
      #define M_HOTRESET               *REG_HOTRESET_PTR = 0x00000000;
   #endif /* M_HOTRESET */
#endif /* DOXYGEN_SHOULD_SKIP_THIS */


   // if the assertion is true, just return
   if(Assertion) return;

   // The framebuffer
   u32 *pFb=MEM_RAM_LOGICAL_PTR;
   int iCurY=30;
   PAD Pad;
   u8 WasStillPressed;

   // Initialize video interface
   VIDEO_Init(VIDEO_640X480_NTSC_YUV16);

   // Set framebuffer address
   VIDEO_SetFrameBuffer(VIDEO_FRAMEBUFFER_BOTH, (u32)pFb);

   // Clear framebuffer
   RENDER_ClearFrameBuffer(pFb, COLOR_NAVY);

   // Init the debug text system
   DEBUG_InitText(pFb, COLOR_WHITE);

   // Display the text now
   DEBUG_Print       (20, iCurY, "*** ASSERTION FAILED ***");  iCurY+=40;

   DEBUG_Print       (20, iCurY, "Message:");                  iCurY+=20;
   DEBUG_Print       (50, iCurY, pMessage);                    iCurY+=130;

   DEBUG_Print       (20, iCurY, "Expression:");               iCurY+=20;
   DEBUG_Print       (50, iCurY, pExpression);                 iCurY+=50;

   DEBUG_Print       (20, iCurY, "In file:");                  iCurY+=20;
   DEBUG_Print       (50, iCurY, pFileName);                   iCurY+=30;

   DEBUG_Print       (20, iCurY, "At line:");                  iCurY+=20;
   DEBUG_ShowValueU32(66, iCurY, LineNo);                      iCurY+=50;

   DEBUG_Print       (20, iCurY, "Press -START- to reset (Pad 0)");

   // read the padstate, to determine if the START button is pressed
   // because we use it as a reset. This is to avoid an immediately
   // reset the assertion screen if START is held when entering
   // the assertion screen.
   PAD_ReadState(&Pad, PAD_CHANNEL_0);
   WasStillPressed = Pad.Digital.Start;

   // infinite loop
   while(1)
   {
      // Read pad state
      PAD_ReadState(&Pad, PAD_CHANNEL_0);

      // Check if start has been pressed
      if(Pad.Digital.Start)
      {
         if(!WasStillPressed)
         {
            // make a reset
            M_HOTRESET;
         }
      }
      else
      {
         WasStillPressed = FALSE;
      }
   }
}
/*+----------------------------------------------------------------------------------------------+*/
/* END OF FILE */
